﻿#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file Spider2020DB.py
 @brief Spider2020 Data store
 @date $Date$


"""
import sys
import datetime
import json
import logging
from logging import DEBUG
import math
import os
from typing import Tuple

# Import RTM module
import RTC
import OpenRTM_aist
import Spider2020DataType_idl

from mongodb import MongoDB
from postgres import PostgreSQL

sys.path.append(".")

log_dir = "./log"
if os.path.isdir(log_dir) is False:
    os.makedirs(log_dir)

if __name__ == '__main__':
    log_format = "%(asctime)s %(levelname)s %(name)s :%(message)s"
    log_file = f"{log_dir}/{datetime.datetime.now(datetime.timezone(datetime.timedelta(hours=+9), 'JST')).strftime('%Y%m%d%H%M%S')}.log"

    # stream handler setting
    stream_handler = logging.StreamHandler()
    stream_handler.setLevel(logging.INFO)
    stream_handler.setFormatter(logging.Formatter(log_format))

    # file handler setting
    file_handler = logging.FileHandler(log_file)
    file_handler.setLevel(DEBUG)
    file_handler.setFormatter(logging.Formatter(log_format))
    logging.basicConfig(level=logging.NOTSET, handlers=[stream_handler, file_handler])

logger = logging.getLogger(__name__)

DB_CONFIG_FILE_PATH = "./db_settings.json"

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
spider2020db_spec = ["implementation_id", "Spider2020DB",
         "type_name",         "Spider2020DB",
         "description",       "Spider2020 Data store",
         "version",           "1.0.0",
         "vendor",            "EAC",
         "category",          "Database",
         "activity_type",     "STATIC",
         "max_instance",      "1",
         "language",          "Python",
         "lang_type",         "SCRIPT",
         ""]
# </rtc-template>

##
# @class Spider2020DB
# @brief Spider2020 Data store
#
#


class Spider2020DB(OpenRTM_aist.DataFlowComponentBase):

    ##
    # @brief constructor
    # @param manager Maneger Object
    #
    def __init__(self, manager):
        OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

        self._d_inAllData = OpenRTM_aist.instantiateDataType(RTC.Spider2020All)
        """
        """
        self._inAllDataIn = OpenRTM_aist.InPort("inAllData", self._d_inAllData)

        # initialize of configuration-data.
        # <rtc-template block="init_conf_param">

        # </rtc-template>

    ##
    #
    # The initialize action (on CREATED->ALIVE transition)
    # formaer rtc_init_entry()
    #
    # @return RTC::ReturnCode_t
    #
    #
    def onInitialize(self):
        # Bind variables and configuration variable

        # Set InPort buffers
        self.addInPort("inAllData", self._inAllDataIn)

        # Set OutPort buffers

        # Set service provider to Ports

        # Set service consumers to Ports

        # Set CORBA Service Ports

        self._mongo_client = None
        self._pg_client = None
        self._pg_cur = None

        return RTC.RTC_OK

    ##
    # DB close, if necessary
    #
    # The finalize action (on ALIVE->END transition)
    # formaer rtc_exiting_entry()
    #
    # @return RTC::ReturnCode_t

    #
    def onFinalize(self):

        self._db_close()

        return RTC.RTC_OK

    ###
    ##
    ## The startup action when ExecutionContext startup
    ## former rtc_starting_entry()
    ##
    ## @param ec_id target ExecutionContext Id
    ##
    ## @return RTC::ReturnCode_t
    ##
    ##
    #def onStartup(self, ec_id):
    #
    #    return RTC.RTC_OK

    ###
    ##
    ## The shutdown action when ExecutionContext stop
    ## former rtc_stopping_entry()
    ##
    ## @param ec_id target ExecutionContext Id
    ##
    ## @return RTC::ReturnCode_t
    ##
    ##
    #def onShutdown(self, ec_id):
    #
    #    return RTC.RTC_OK

    ##
    # DB Authentication
    #
    # The activated action (Active state entry action)
    # former rtc_active_entry()
    #
    # @param ec_id target ExecutionContext Id
    #
    # @return RTC::ReturnCode_t
    #
    #
    def onActivated(self, ec_id):

        result, db_conf = self._is_json_file_parse(DB_CONFIG_FILE_PATH)
        if result is False:
            return RTC.RTC_ERROR

        # Create a DB instance
        # Mongo
        self._mongo_ins = MongoDB(db_conf["1st_db_host"], db_conf["1st_db_port"], db_conf["1st_db_name"], db_conf["1st_db_tbl"])
        self._mongo_client = self._mongo_ins.connection()
        # PostgreSQL
        self._pg_ins = PostgreSQL(db_conf["2nd_db_host"], db_conf["2nd_db_port"], db_conf["2nd_db_name"], db_conf["2nd_db_tbl"], db_conf["2nd_db_user"], db_conf["2nd_db_pass"])
        self._pg_client = self._pg_ins.get_connection()
        self._pg_cur = self._pg_ins.get_acquisition_cursor(self._pg_client)

        if self._mongo_client and self._pg_client:
            return RTC.RTC_OK
        else:
            return RTC.RTC_ERROR

    ##
    # DB close, if necessary
    #
    # The deactivated action (Active state exit action)
    # former rtc_active_exit()
    #
    # @param ec_id target ExecutionContext Id
    #
    # @return RTC::ReturnCode_t
    #
    #
    def onDeactivated(self, ec_id):

        self._db_close()

        return RTC.RTC_OK

    ##
    # Data write to DB
    #
    # The execution action that is invoked periodically
    # former rtc_active_do()
    #
    # @param ec_id target ExecutionContext Id
    #
    # @return RTC::ReturnCode_t
    #
    #
    def onExecute(self, ec_id):

        if self._inAllDataIn.isNew():
            logger.info("Received: " + datetime.datetime.now(datetime.timezone(datetime.timedelta(hours=+9), 'JST')).strftime('%Y-%m-%d %H:%M:%S.%f'))
            self._d_inAllData = self._inAllDataIn.read()
            logger.info("Robot Datetime: " + self._rtctime2strdatetime(self._d_inAllData.robotid.tm.sec, self._d_inAllData.robotid.tm.nsec))
            # print(self._d_inAllData)
            # Inset to Mongo
            self._insert_mongo(self._to_json())
            # Insert to PostgreSQL
            self._spider2020_conv()
            self._insert_pg()

        return RTC.RTC_OK

    ###
    ##
    ## The aborting action when main logic error occurred.
    ## former rtc_aborting_entry()
    ##
    ## @param ec_id target ExecutionContext Id
    ##
    ## @return RTC::ReturnCode_t
    ##
    ##
    #def onAborting(self, ec_id):
    #
    #    return RTC.RTC_OK

    ##
    # DB close, if necessary
    #
    # The error action in ERROR state
    # former rtc_error_do()
    #
    # @param ec_id target ExecutionContext Id
    #
    # @return RTC::ReturnCode_t
    #
    #
    def onError(self, ec_id):

        self._db_close()
        self.reset(ec_id)

        return RTC.RTC_OK

    ##
    # DB close, if necessary
    #
    # The reset action that is invoked resetting
    # This is same but different the former rtc_init_entry()
    #
    # @param ec_id target ExecutionContext Id
    #
    # @return RTC::ReturnCode_t
    #
    #
    def onReset(self, ec_id):

        self._db_close()

        return RTC.RTC_OK

    ###
    ##
    ## The state update action that is invoked after onExecute() action
    ## no corresponding operation exists in OpenRTm-aist-0.2.0
    ##
    ## @param ec_id target ExecutionContext Id
    ##
    ## @return RTC::ReturnCode_t
    ##

    ##
    #def onStateUpdate(self, ec_id):
    #
    #    return RTC.RTC_OK

    ###
    ##
    ## The action that is invoked when execution context's rate is changed
    ## no corresponding operation exists in OpenRTm-aist-0.2.0
    ##
    ## @param ec_id target ExecutionContext Id
    ##
    ## @return RTC::ReturnCode_t
    ##
    ##
    #def onRateChanged(self, ec_id):
    #
    #    return RTC.RTC_OK

    def _is_json_file_parse(self, path):
        """
        Read to specified json file.

        Parameters
        ----------
        Path
            json file path

        Returns
        -------
        Parse result
            True: Success False: Failure
        """

        result = False
        json_dict = {}

        try:
            with open(path) as f:
                json_dict = json.load(f)
                if "" not in json_dict.values():
                    result = True
                else:
                    logger.error("Param is empty.Please confirm.")
        except IOError:
            logger.error("Unable to read setting file.")

        return result, json_dict

    def _to_json(self):
        """
        Receive data formatting to json.

        Parameters
        ----------
        None

        Returns
        -------
        Json data
        """

        all_data = {
            "robot_id": self._d_inAllData.robotid.data,
            "robot_id_datetime": self._rtctime2strdatetime(self._d_inAllData.robotid.tm.sec, self._d_inAllData.robotid.tm.nsec),
            "pose3d": {
                "datetime": self._rtctime2strdatetime(self._d_inAllData.pose3d.tm.sec, self._d_inAllData.pose3d.tm.nsec),
                "position": {
                    "x": self._d_inAllData.pose3d.data.position.x,
                    "y": self._d_inAllData.pose3d.data.position.y,
                    "z": self._d_inAllData.pose3d.data.position.z
                },
                "orientation": {
                    "roll": self._d_inAllData.pose3d.data.orientation.r,
                    "pitch": self._d_inAllData.pose3d.data.orientation.p,
                    "yaw": self._d_inAllData.pose3d.data.orientation.y
                }
            },
            "motor_load": {
                "datetime": self._rtctime2strdatetime(self._d_inAllData.motorLoad.tm.sec, self._d_inAllData.motorLoad.tm.nsec),
                "main_r": self._d_inAllData.motorLoad.data[0],
                "main_l": self._d_inAllData.motorLoad.data[1],
                "sub_fr": self._d_inAllData.motorLoad.data[2],
                "sub_fl": self._d_inAllData.motorLoad.data[3],
                "sub_br": self._d_inAllData.motorLoad.data[4],
                "sub_bl": self._d_inAllData.motorLoad.data[5]
            },
            "flipper_angle": {
                "datetime": self._rtctime2strdatetime(self._d_inAllData.flipperAngle.tm.sec, self._d_inAllData.flipperAngle.tm.nsec),
                "fr": self._d_inAllData.flipperAngle.data[0],
                "fl": self._d_inAllData.flipperAngle.data[1],
                "br": self._d_inAllData.flipperAngle.data[2],
                "bl": self._d_inAllData.flipperAngle.data[3]
            },
            "imu_orientation3d": {
                "datetime": self._rtctime2strdatetime(self._d_inAllData.imuOri3d.tm.sec, self._d_inAllData.imuOri3d.tm.nsec),
                "roll": self._d_inAllData.imuOri3d.data.r,
                "pitch": self._d_inAllData.imuOri3d.data.p,
                "yaw": self._d_inAllData.imuOri3d.data.y
            },
            "velocity2d": {
                "dateimte": self._rtctime2strdatetime(self._d_inAllData.velo2d.tm.sec, self._d_inAllData.velo2d.tm.nsec),
                "vx": self._d_inAllData.velo2d.data.vx,
                "vy": self._d_inAllData.velo2d.data.vy,
                "va": self._d_inAllData.velo2d.data.va
            },
            "gc_capacity_ratio": {
                "datetime": self._rtctime2strdatetime(self._d_inAllData.gcCapaRatio.tm.sec, self._d_inAllData.gcCapaRatio.tm.nsec),
                "val": self._d_inAllData.gcCapaRatio.data
            },
            "gc_axis": {
                "datetime": self._rtctime2strdatetime(self._d_inAllData.gcAxis.tm.sec, self._d_inAllData.gcAxis.tm.nsec),
                "direction_h": self._d_inAllData.gcAxis.data[0],
                "direction_v": self._d_inAllData.gcAxis.data[1],
                "lstick_h": self._d_inAllData.gcAxis.data[2],
                "lstick_v": self._d_inAllData.gcAxis.data[3],
                "rstick_h": self._d_inAllData.gcAxis.data[4],
                "rstick_v": self._d_inAllData.gcAxis.data[5],
                "l2": self._d_inAllData.gcAxis.data[6],
                "r2": self._d_inAllData.gcAxis.data[7],
            },
            "gc_button": {
                "datetime": self._rtctime2strdatetime(self._d_inAllData.gcButton.tm.sec, self._d_inAllData.gcButton.tm.nsec),
                # "button_square": self._d_inAllData.gcButton.data & 0b00000000000000000000000000000001,
                # "button_cross": (self._d_inAllData.gcButton.data & 0b00000000000000000000000000000010) >> 1,
                # "button_circle": (self._d_inAllData.gcButton.data & 0b00000000000000000000000000000100) >> 2,
                # "button_triangle": (self._d_inAllData.gcButton.data & 0b00000000000000000000000000001000) >> 3,
                # "button_l1": (self._d_inAllData.gcButton.data & 0b00000000000000000000000000010000) >> 4,
                # "button_r1": (self._d_inAllData.gcButton.data & 0b00000000000000000000000000100000) >> 5,
                # "button_l2": (self._d_inAllData.gcButton.data & 0b00000000000000000000000001000000) >> 6,
                # "button_r2": (self._d_inAllData.gcButton.data & 0b00000000000000000000000010000000) >> 7,
                # "button_share": (self._d_inAllData.gcButton.data & 0b00000000000000000000000100000000) >> 8,
                # "button_options": (self._d_inAllData.gcButton.data & 0b00000000000000000000001000000000) >> 9,
                # "button_l3": (self._d_inAllData.gcButton.data & 0b00000000000000000000010000000000) >> 10,
                # "button_r3": (self._d_inAllData.gcButton.data & 0b00000000000000000000100000000000) >> 11,
                # "button_ps": (self._d_inAllData.gcButton.data & 0b00000000000000000001000000000000) >> 12,
                # "button_touch": (self._d_inAllData.gcButton.data & 0b00000000000000000010000000000000) >> 13
                "button_square": self._d_inAllData.gcButton.data,
                "button_cross": self._d_inAllData.gcButton.data,
                "button_circle": self._d_inAllData.gcButton.data,
                "button_triangle": self._d_inAllData.gcButton.data,
                "button_l1": self._d_inAllData.gcButton.data,
                "button_r1": self._d_inAllData.gcButton.data,
                "button_l2": self._d_inAllData.gcButton.data,
                "button_r2": self._d_inAllData.gcButton.data,
                "button_share": self._d_inAllData.gcButton.data,
                "button_options": self._d_inAllData.gcButton.data,
                "button_l3": self._d_inAllData.gcButton.data,
                "button_r3": self._d_inAllData.gcButton.data,
                "button_ps": self._d_inAllData.gcButton.data,
                "button_touch": self._d_inAllData.gcButton.data
            },
            "main_capacity_ratio": {
                "datetime": self._rtctime2strdatetime(self._d_inAllData.mainBattCapaRatio.tm.sec, self._d_inAllData.mainBattCapaRatio.tm.nsec),
                "val": self._d_inAllData.mainBattCapaRatio.data
            },
            "main_supply_voltage": {
                "datetime": self._rtctime2strdatetime(self._d_inAllData.mainSupplyVolt.tm.sec, self._d_inAllData.mainSupplyVolt.tm.nsec),
                "val": self._d_inAllData.mainSupplyVolt.data
            },
            "main_raw_supply_voltage": {
                "datetime": self._rtctime2strdatetime(self._d_inAllData.mainRawSupplyVolt.tm.sec, self._d_inAllData.mainRawSupplyVolt.tm.nsec),
                "val": self._d_inAllData.mainRawSupplyVolt.data
            },
            "mode": {
                "datetime": self._rtctime2strdatetime(self._d_inAllData.mode.tm.sec, self._d_inAllData.mode.tm.nsec),
                "action": self._d_inAllData.mode.data[0],
                "speed_level": self._d_inAllData.mode.data[1],
                "speed_max": self._d_inAllData.mode.data[2],
                # "flipper_fr": self._d_inAllData.mode.data[3],
                # "flipper_fl": self._d_inAllData.mode.data[4],
                # "flipper_br": self._d_inAllData.mode.data[5],
                # "flipper_bl": self._d_inAllData.mode.data[6],
                # "flipper_action": self._d_inAllData.mode.data[7],
                # "pose_clear": self._d_inAllData.mode.data[8]
                "flipper": self._d_inAllData.mode.data[3],
                "flipper_action": self._d_inAllData.mode.data[4],
                "pose_clear": self._d_inAllData.mode.data[5]
            },
            "arm1_pose3d": {
                "datetime": self._rtctime2strdatetime(self._d_inAllData.arm1Pose3d.tm.sec, self._d_inAllData.arm1Pose3d.tm.nsec),
                "position": {
                    "x": self._d_inAllData.arm1Pose3d.data.position.x,
                    "y": self._d_inAllData.arm1Pose3d.data.position.y,
                    "z": self._d_inAllData.arm1Pose3d.data.position.z
                },
                "orientation": {
                    "roll": self._d_inAllData.arm1Pose3d.data.orientation.r,
                    "pitch": self._d_inAllData.arm1Pose3d.data.orientation.p,
                    "yaw": self._d_inAllData.arm1Pose3d.data.orientation.y
                }
            },
            "arm1_angular": {
                "datetime": self._rtctime2strdatetime(self._d_inAllData.arm1Angular.tm.sec, self._d_inAllData.arm1Angular.tm.nsec),
                "angular": [self._d_inAllData.arm1Angular.data[0], self._d_inAllData.arm1Angular.data[1], self._d_inAllData.arm1Angular.data[2], self._d_inAllData.arm1Angular.data[3], self._d_inAllData.arm1Angular.data[4], self._d_inAllData.arm1Angular.data[5]]
            },
            "arm1_finger": {
                "datetime": self._rtctime2strdatetime(self._d_inAllData.arm1Finger.tm.sec, self._d_inAllData.arm1Finger.tm.nsec),
                "finger": [self._d_inAllData.arm1Finger.data[0], self._d_inAllData.arm1Finger.data[1], self._d_inAllData.arm1Finger.data[2]]
            },
            "arm1_current": {
                "datetime": self._rtctime2strdatetime(self._d_inAllData.arm1Current.tm.sec, self._d_inAllData.arm1Current.tm.nsec),
                "angular": [self._d_inAllData.arm1Current.data[0], self._d_inAllData.arm1Current.data[1], self._d_inAllData.arm1Current.data[2], self._d_inAllData.arm1Current.data[3], self._d_inAllData.arm1Current.data[4], self._d_inAllData.arm1Current.data[5]],
                "finger": [self._d_inAllData.arm1Current.data[6], self._d_inAllData.arm1Current.data[7], self._d_inAllData.arm1Current.data[8]]
            }
        }

        return all_data

    def _rtctime2strdatetime(self, sec: int, nsec: int) -> str:
        """
        RTC time to Datetime(yyyy-MM-dd HH:mm:ss.f).

        Parameters
        ----------
        RTC time

        Returns
        -------
        Datetime for string(yyyy-MM-dd HH:mm:ss.f)
        """

        rtc2strtime = str(sec) + "." + str(nsec)
        str_time = datetime.datetime.fromtimestamp(float(rtc2strtime)).strftime("%Y-%m-%d %H:%M:%S.%f")

        return str_time

    def _insert_mongo(self, json_str) -> bool:
        """
        Insert to MongoDB.

        Parameters
        ----------
        json_str
            Insert data

        Returns
        -------
        Insert Result
        """

        rtn = False

        if self._mongo_ins.insert_document(self._mongo_ins._db_name, self._mongo_ins._collection_name, json_str):
            rtn = True

        return rtn

    def _spider2020_conv(self):
        """
        Data Convert

        Parameters
        ----------
        None

        Returns
        -------
        None
        """

        self._pose3d_x, self._pose3d_y, self._pose3d_z, self._pose3d_w = self._euler2quat(self._d_inAllData.pose3d.data.orientation.r, self._d_inAllData.pose3d.data.orientation.p, self._d_inAllData.pose3d.data.orientation.y)
        self._imu3d_x, self._imu3d_y, self._imu3d_z, self._imu3d_w = self._euler2quat(self._d_inAllData.imuOri3d.data.r, self._d_inAllData.imuOri3d.data.p, self._d_inAllData.imuOri3d.data.y)
        self._arm1_pose3d_x, self._arm1_pose3d_y, self._arm1_pose3d_z, self._arm1_pose3d_w = self._euler2quat(self._d_inAllData.arm1Pose3d.data.orientation.r, self._d_inAllData.arm1Pose3d.data.orientation.p, self._d_inAllData.arm1Pose3d.data.orientation.y)

    def _euler2quat(self, roll: float, pitch: float, yaw: float) -> Tuple[float, float, float, float]:
        """
        Euler --> Quaternion

        Parameters
        ----------
        Euler(roll, pitch, yaw)

        Returns
        -------
        Quaternion(x, y, z, w)
        """

        cos_r = math.cos(roll / 2.0)
        sin_r = math.sin(roll / 2.0)
        cos_p = math.cos(pitch / 2.0)
        sin_p = math.sin(pitch / 2.0)
        cos_y = math.cos(yaw / 2.0)
        sin_y = math.sin(yaw / 2.0)

        x = sin_r * cos_p * cos_y - cos_r * sin_p * sin_y
        y = sin_r * cos_p * sin_y + cos_r * sin_p * cos_y
        z = - sin_r * sin_p * cos_y + cos_r * cos_p * sin_y
        w = sin_r * sin_p * sin_y + cos_r * cos_p * cos_y

        return x, y, z, w

    def _insert_pg(self) -> bool:
        """
        Insert to PostgreSQL.

        Parameters
        ----------
        None

        Returns
        -------
        Insert Result
        """

        rtn = False

        try:
            self._insert_pose3d()
            self._insert_motor_load()
            self._insert_flipper_angle()
            self._insert_imu_ori3d()
            self._insert_velo2d()
            self._insert_gc()
            self._insert_main_batt()
            self._insert_mode()
            self._insert_arm1_pose3d()
            self._insert_arm1_angular()
            self._insert_arm1_finger()
            self._insert_arm1_current()
            self._insert_spider2020()

            self._pg_ins._commit()
            rtn = True
        except Exception:
            self._pg_ins._rollback()

        return rtn

    def _insert_pose3d(self):
        """
        Insert to Pose3D

        Parameters
        ----------
        None

        Returns
        -------
        None
        """

        # self._pg_ins._tbl_name = "pose3d"
        self._pg_ins._tbl_name = "pose_3d"
        INSERT_POSE3D = "INSERT INTO {} (x, y, z, roll, pitch, yaw, quat_x, quat_y, quat_z, quat_w) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s)"
        param = (self._d_inAllData.pose3d.data.position.x, self._d_inAllData.pose3d.data.position.y, self._d_inAllData.pose3d.data.position.z,
                 self._d_inAllData.pose3d.data.orientation.r, self._d_inAllData.pose3d.data.orientation.p, self._d_inAllData.pose3d.data.orientation.y,
                 self._pose3d_x, self._pose3d_y, self._pose3d_z, self._pose3d_w)
        self._pg_ins.insert(self._pg_cur, INSERT_POSE3D, param)

    def _insert_motor_load(self):
        """
        Insert to MotorLoad

        Parameters
        ----------
        None

        Returns
        -------
        None
        """

        self._pg_ins._tbl_name = "motor_load"
        INSERT_MOTOR_LOAD = "INSERT INTO {} (main_r, main_l, sub_fr, sub_fl, sub_br, sub_bl) VALUES (%s, %s, %s, %s, %s, %s)"
        param = (self._d_inAllData.motorLoad.data[0], self._d_inAllData.motorLoad.data[1], self._d_inAllData.motorLoad.data[2],
                 self._d_inAllData.motorLoad.data[3], self._d_inAllData.motorLoad.data[4], self._d_inAllData.motorLoad.data[5])
        self._pg_ins.insert(self._pg_cur, INSERT_MOTOR_LOAD, param)

    def _insert_flipper_angle(self):
        """
        Insert to Flipper Angle

        Parameters
        ----------
        None

        Returns
        -------
        None
        """

        self._pg_ins._tbl_name = "flipper_angle"
        FLIPPER_ANGLE = "INSERT INTO {} (fr, fl, br, bl) VALUES (%s, %s, %s, %s)"
        param = (self._d_inAllData.flipperAngle.data[0], self._d_inAllData.flipperAngle.data[1],
                 self._d_inAllData.flipperAngle.data[2], self._d_inAllData.flipperAngle.data[3])
        self._pg_ins.insert(self._pg_cur, FLIPPER_ANGLE, param)

    def _insert_imu_ori3d(self):
        """
        Insert to IMU Orientation 3D

        Parameters
        ----------
        None

        Returns
        -------
        None
        """

        self._pg_ins._tbl_name = "imu_pose3d"
        IMU3D = "INSERT INTO {} (roll, pitch, yaw, quat_x, quat_y, quat_z, quat_w) VALUES (%s, %s, %s, %s, %s, %s, %s)"
        param = (self._d_inAllData.imuOri3d.data.r, self._d_inAllData.imuOri3d.data.p, self._d_inAllData.imuOri3d.data.y,
                 self._imu3d_x, self._imu3d_y, self._imu3d_z, self._imu3d_w)
        self._pg_ins.insert(self._pg_cur, IMU3D, param)

    def _insert_velo2d(self):
        """
        Insert to Velocity 2D

        Parameters
        ----------
        None

        Returns
        -------
        None
        """

        self._pg_ins._tbl_name = "velocity"
        VELO2D = "INSERT INTO {} (vx, vy, va) VALUES (%s, %s, %s)"
        param = (self._d_inAllData.velo2d.data.vx, self._d_inAllData.velo2d.data.vy, self._d_inAllData.velo2d.data.va)
        self._pg_ins.insert(self._pg_cur, VELO2D, param)

    def _insert_gc(self):
        """
        Insert to Game Controller status

        Parameters
        ----------
        None

        Returns
        -------
        None
        """

        self._pg_ins._tbl_name = "game_controller"
        GC = "INSERT INTO {} (capacity_ratio_val, \
                              axis_direction_h, axis_direction_v, axis_lstick_h, axis_lstick_v, axis_rstick_h, axis_rstick_v, axis_l2, axis_r2, \
                              button_square, button_cross, button_circle, button_triangle, button_l1, button_r1, button_l2, button_r2, \
                              button_share, button_options, button_l3, button_r3, button_ps, button_touch) \
                              VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)"
        param = (self._d_inAllData.gcCapaRatio.data,
                 self._d_inAllData.gcAxis.data[0], self._d_inAllData.gcAxis.data[1], self._d_inAllData.gcAxis.data[2], self._d_inAllData.gcAxis.data[3],
                 self._d_inAllData.gcAxis.data[4], self._d_inAllData.gcAxis.data[5], self._d_inAllData.gcAxis.data[6], self._d_inAllData.gcAxis.data[7],
                 self._d_inAllData.gcButton.data, self._d_inAllData.gcButton.data, self._d_inAllData.gcButton.data, self._d_inAllData.gcButton.data, self._d_inAllData.gcButton.data,
                 self._d_inAllData.gcButton.data, self._d_inAllData.gcButton.data, self._d_inAllData.gcButton.data, self._d_inAllData.gcButton.data, self._d_inAllData.gcButton.data,
                 self._d_inAllData.gcButton.data, self._d_inAllData.gcButton.data, self._d_inAllData.gcButton.data, self._d_inAllData.gcButton.data)
        self._pg_ins.insert(self._pg_cur, GC, param)

    def _insert_main_batt(self):
        """
        Insert to Pose3D

        Parameters
        ----------
        None

        Returns
        -------
        None
        """

        self._pg_ins._tbl_name = "main_batt"
        BATT = "INSERT INTO {} (capacity_ratio_val, supply_voltage_val, raw_supply_voltage_val) VALUES (%s, %s, %s)"
        param = (self._d_inAllData.mainBattCapaRatio.data, self._d_inAllData.mainSupplyVolt.data, self._d_inAllData.mainRawSupplyVolt.data)
        self._pg_ins.insert(self._pg_cur, BATT, param)

    def _insert_mode(self):
        """
        Insert to Mode

        Parameters
        ----------
        None

        Returns
        -------
        None
        """

        self._pg_ins._tbl_name = "mode"
        MODE = "INSERT INTO {} (action, speed_level, speed_max, flipper_select_fr, flipper_select_fl, flipper_select_br, flipper_select_bl, flipper_action, pose_clear) \
                                VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s)"
        # param = (self._d_inAllData.mode.data[0], self._d_inAllData.mode.data[1], self._d_inAllData.mode.data[2], self._d_inAllData.mode.data[3],
        #          self._d_inAllData.mode.data[4], self._d_inAllData.mode.data[5], self._d_inAllData.mode.data[6], self._d_inAllData.mode.data[7], self._d_inAllData.mode.data[8])
        param = (self._d_inAllData.mode.data[0], self._d_inAllData.mode.data[1], self._d_inAllData.mode.data[2], self._d_inAllData.mode.data[3],
                 self._d_inAllData.mode.data[3], self._d_inAllData.mode.data[3], self._d_inAllData.mode.data[3], self._d_inAllData.mode.data[4], self._d_inAllData.mode.data[5])
        self._pg_ins.insert(self._pg_cur, MODE, param)

    def _insert_arm1_pose3d(self):
        """
        Insert to Arm1 Pose3D

        Parameters
        ----------
        None

        Returns
        -------
        None
        """

        # self._pg_ins._tbl_name = "arm1_pose3d"
        self._pg_ins._tbl_name = "arm_pose"
        INSERT_ARM1_POSE3D = "INSERT INTO {} (x, y, z, roll, pitch, yaw, quat_x, quat_y, quat_z, quat_w) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s)"
        param = (self._d_inAllData.arm1Pose3d.data.position.x, self._d_inAllData.arm1Pose3d.data.position.y, self._d_inAllData.arm1Pose3d.data.position.z,
                 self._d_inAllData.arm1Pose3d.data.orientation.r, self._d_inAllData.arm1Pose3d.data.orientation.p, self._d_inAllData.arm1Pose3d.data.orientation.y,
                 self._arm1_pose3d_x, self._arm1_pose3d_y, self._arm1_pose3d_z, self._arm1_pose3d_w)
        self._pg_ins.insert(self._pg_cur, INSERT_ARM1_POSE3D, param)

    def _insert_arm1_angular(self):
        """
        Insert to Arm1 Angular

        Parameters
        ----------
        None

        Returns
        -------
        None
        """

        # self._pg_ins._tbl_name = "arm1_angular"
        self._pg_ins._tbl_name = "arm_angular"
        INSERT_ARM1_ANGULAR = "INSERT INTO {} (angular1, angular2, angular3, angular4, angular5, angular6) VALUES (%s, %s, %s, %s, %s, %s)"
        param = (self._d_inAllData.arm1Angular.data[0], self._d_inAllData.arm1Angular.data[1], self._d_inAllData.arm1Angular.data[2],
                 self._d_inAllData.arm1Angular.data[3], self._d_inAllData.arm1Angular.data[4], self._d_inAllData.arm1Angular.data[5])
        self._pg_ins.insert(self._pg_cur, INSERT_ARM1_ANGULAR, param)

    def _insert_arm1_finger(self):
        """
        Insert to Arm1 Finger

        Parameters
        ----------
        None

        Returns
        -------
        None
        """

        # self._pg_ins._tbl_name = "arm1_finger"
        self._pg_ins._tbl_name = "arm_finger"
        INSERT_ARM1_FINGER = "INSERT INTO {} (finger1, finger2, finger3) VALUES (%s, %s, %s)"
        param = (self._d_inAllData.arm1Finger.data[0], self._d_inAllData.arm1Finger.data[1], self._d_inAllData.arm1Finger.data[2])
        self._pg_ins.insert(self._pg_cur, INSERT_ARM1_FINGER, param)

    def _insert_arm1_current(self):
        """
        Insert to Arm1 Current

        Parameters
        ----------
        None

        Returns
        -------
        None
        """

        # self._pg_ins._tbl_name = "arm1_current"
        self._pg_ins._tbl_name = "arm_current"
        INSERT_ARM1_CURRENT = "INSERT INTO {} (angular1, angular2, angular3, angular4, angular5, angular6, finger1, finger2, finger3) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s)"
        param = (self._d_inAllData.arm1Current.data[0], self._d_inAllData.arm1Current.data[1], self._d_inAllData.arm1Current.data[2],
                 self._d_inAllData.arm1Current.data[3], self._d_inAllData.arm1Current.data[4], self._d_inAllData.arm1Current.data[5],
                 self._d_inAllData.arm1Current.data[6], self._d_inAllData.arm1Current.data[7], self._d_inAllData.arm1Current.data[8])
        self._pg_ins.insert(self._pg_cur, INSERT_ARM1_CURRENT, param)

    def _insert_spider2020(self):
        """
        Insert to RobotID and Datetime

        Parameters
        ----------
        None

        Returns
        -------
        None
        """

        self._pg_ins._tbl_name = "spider2020"
        INSERT_SPIDER2020 = "INSERT INTO {} (robot_id, datetime) VALUES (%s, %s)"
        param = (self._d_inAllData.robotid.data, self._rtctime2strdatetime(self._d_inAllData.robotid.tm.sec, self._d_inAllData.robotid.tm.nsec))
        self._pg_ins.insert(self._pg_cur, INSERT_SPIDER2020, param)

    def _db_close(self):
        """
        DB close

        Parameters
        ----------
        None

        Returns
        -------
        None
        """

        if self._mongo_client:
            self._mongo_ins.close()
            self._mongo_client = None

        if self._pg_client:
            self._pg_ins.close()
            self._pg_cur = None
            self._pg_client = None


def Spider2020DBInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=spider2020db_spec)
    manager.registerFactory(profile,
                            Spider2020DB,
                            OpenRTM_aist.Delete)


def MyModuleInit(manager):
    Spider2020DBInit(manager)

    # Create a component
    comp = manager.createComponent("Spider2020DB")


def main():
    mgr = OpenRTM_aist.Manager.init(sys.argv)
    mgr.setModuleInitProc(MyModuleInit)
    mgr.activateManager()
    mgr.runManager()


if __name__ == "__main__":
    main()
